% Example 1: Recover sparse signal via Fourier matrix and l-1 minimization
% This script demonstrates the ability to simultaneously recover noiseless
% sparse support and its nonzero coefficients using l-1 minimization.
%
% Matlab toolbox: cvx
% Example due to E. Candes.
% 
% Allen Y. Yang. Feb. 2008, UC Berkeley

clear;
close all;

n = 512;    % size of signal
S = 28;     % Sparsity

support = randsample(n,S);
x0 = zeros(n,1); x0(support) = randn(S,1);
figure(1); hold on
plot(1:n,x0,'b*');

% Sampling
m = 64;
k=0:(n-1); t=0:(n-1);
F = exp(-j*2*pi*(k'*t)/n)/sqrt(n);  % Fourier matrix
freq = randsample(n,m);
A = [real(F(freq,:)); imag(F(freq,:))];

% Project
y = A*x0;
figure(2); 
plot(y);

% l-1 minimization
cvx_begin
    variable x(n);
    minimize(norm(x,1));
    subject to
        A*x == y;
cvx_end
figure(1);
plot(1:n,x,'ro');